require "urbans/version"

require "yaml"

# exceptions
require "urbans/exceptions/urban_error"

# foundations
require "urbans/foundations/city"
require "urbans/foundations/country"
require "urbans/foundations/province"

require "urbans/configurer"

module Urbans
  URBAN_PATH = File.expand_path(File.dirname(__FILE__)) + "/"

  # all avalable countries supported by Urbans
  ALL_COUNTRIES = [:id]

  # all available locales
  ALL_LOCALES = [:id, :en]
  @@default_locale = nil

  # mapping of countries and locales, eg:
  # {
  #   id: {en: Indonesia, id: Indonesia}
  # }
  # only load locale when necessary
  COUNTRIES = {}

  # mapping of provinces and locales, eg:
  # {
  #   id_pv_ach: {en: "Aceh", id: "Aceh"}
  # }
  # only load locale when necessary
  PROVINCES = {}

  # similar to above
  CITIES = {}

  module_function

  def config(&block)
    Urbans::Configurer.new.instance_eval(&block)
  end

  def locale=(locale)
    if ALL_LOCALES.include?(locale.to_sym)
      @@default_locale = locale.to_sym
    else
      raise Urbans::Error, "Unsupported locale: #{locale}."
    end
  end

  def locale
    raise Urbans::Error, "Locale have not been set yet" if @@default_locale.nil?
    @@default_locale
  end

  def country
    Urbans::Country
  end

  def province
    Urbans::Province
  end

  def city
    Urbans::City
  end

end
