class Urbans::Country
  # raw id from file, useful only for things to do that is related with file
  attr_accessor :__id__

  # what the user need to know about the id
  attr_accessor :id

  # by locale => id: {id: Indonesia, en: Indonesia}
  attr_accessor :names

  def initialize
    self.names = {}
  end

  # return all loaded countries
  def self.all locale=Urbans.locale
    return Urbans::COUNTRIES.values
  end

  def self.all! locale=Urbans.locale
    countries = []
    from_file(locale).each do |country_id, country_data|
      country = Urbans::COUNTRIES[country_id.to_s.downcase.to_sym]
      if country.nil?
        country = Urbans::Country.new
        country.__id__ = country_id
        country.id = country_id.to_s.downcase
        country.names[locale] = country_data["name"]
        Urbans::COUNTRIES[country.id.to_sym] = country
      end
      countries << country
    end
    countries
  end

  # return instance of a country
  def self.get country_id
    if country_id.is_a?(Hash)
      country_id = country_id[:country]
    end

    country = Urbans::COUNTRIES[country_id.to_sym]
    if country.nil?
      locale = Urbans.locale
      country_from_file = from_file(locale)[country_id.to_s.upcase]

      # create a new country instance
      country = Urbans::Country.new
      # ID of a country in file is always in uppercase
      country.__id__ = country_id.to_s.upcase
      country.id = country_id.to_s.downcase
      country.names[locale.to_sym] = country_from_file["name"]

      # keep in hash
      Urbans::COUNTRIES[country_id.to_sym] = country
    else
      return country
    end
  end

  def provinces
    Urbans.province.get country: id
  end

  def cities_by_province

  end

  def name _locale=Urbans.locale
    locale = _locale.is_a?(Symbol) ? _locale : _locale.to_s.downcase.to_sym

    if names[locale].nil?
      # try to load
      local_name = self.class.from_file(locale)[__id__]["name"]
      names[locale] = local_name
    end

    names[locale]
  end

  def <=> another
    self.name <=> another.name
  end

  def self.from_file locale
    from_file = YAML.load_file(Urbans::URBAN_PATH + "Urbans/data/#{locale}.countries.yml")
    from_file[locale.to_s]["countries"]
  end
end
