class Urbans::Province
  attr_accessor :__id__
  attr_accessor :id
  attr_accessor :names
  attr_accessor :country_id

  def initialize
    self.names = {}
  end

  def self.all! _country_id, _locale=Urbans.locale
    provinces = []

    country_id = _country_id.to_s.downcase.to_sym
    locale = _locale.to_s.downcase.to_sym

    from_file(country_id, locale).each do |_province_id, province_name|
      province_id = _province_id.to_s.downcase.to_sym

      Urbans::PROVINCES[country_id] ||= {}
      province = Urbans::PROVINCES[country_id][province_id]
      if province.nil?
        province = Urbans::Province.new
        province.__id__ = _province_id
        province.id = "#{country_id}.#{province_id}"
        province.names[locale] = province_name
        province.country_id = country_id
        Urbans::PROVINCES[country_id][province_id] = province
      end
      provinces << province
    end
    provinces
  end

  # get specific province
  def self.get arg
    _country_id = _province_id = nil
    if arg.is_a?(Hash)
      _country_id, _province_id = arg.fetch(:country), arg[:province]
    elsif arg.is_a?(String)
      _arg = arg.split(".")
      _country_id, _province_id = _arg[0], _arg[1]
    end

    if _province_id.nil?
      return all! _country_id
    end

    country_id = _country_id.to_s.downcase.to_sym
    province_id = _province_id.to_s.downcase.to_sym

    if Urbans::PROVINCES[country_id].nil? || Urbans::PROVINCES[country_id][province_id].nil?
      all! country_id, Urbans.locale
    end

    province = Urbans::PROVINCES[country_id][province_id]
    province
  end

  def name _locale=Urbans.locale
    locale = _locale.to_s.downcase.to_sym

    if names[locale].nil?
      province_local_name = self.class.from_file(country_id, locale)[__id__]
      names[locale] = province_local_name
    end

    names[locale]
  end

  def country
    Urbans.country.get country_id
  end

  def cities
    Urbans.city.get country: country_id, province: __id__
  end

  # for sorting
  def <=> another
    self.name <=> another.name
  end

  def self.from_file _country_id, _locale
    locale = _locale.to_s.downcase
    country_id = _country_id.to_s.downcase
    from_file = YAML.load_file(Urbans::URBAN_PATH + "urbans/data/#{locale}.#{country_id}.provinces.yml")
    from_file[locale]["provinces"][country_id]
  end
end
